/* 
 * Driver commands for Marvell WLAN driver 
 * 
 * Added by Li Yanbing, Sep 16, 2008
 */

#include "wlan_adp_intf.h"
#include "wlan_priv.h"
#include "wireless_copy.h"
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <sys/ioctl.h>
#include <net/if_arp.h>
#include <stdlib.h>
#include <linux/in.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

static struct wpa_driver_wext_data * gDrvData = NULL;

#define MAC2STR(a) (a)[0], (a)[1], (a)[2], (a)[3], (a)[4], (a)[5]
#define MACSTR "%02x:%02x:%02x:%02x:%02x:%02x"

#define wpa_printf v_printf

#define MSG_MSGDUMP VMSG_MSGDUMP
#define MSG_DEBUG VMSG_DEBUG
#define MSG_INFO VMSG_INFO
#define MSG_WARNING VMSG_WARNING
#define MSG_ERROR VMSG_ERROR

enum {
    ANDROID_DRIVER_POWER_MODE_AUTO      = 0,
    ANDROID_DRIVER_POWER_MODE_ACTIVE    = 1,
    ANDROID_DRIVER_POWER_MODE_DEEPSLEEP = 2,
    ANDROID_DRIVER_POWER_MODE_DEEPSLEEP_EXIT = 3,
    ANDROID_DRIVER_POWER_MODE_AUTO_DEEPSLEEP = 4,
    ANDROID_DRIVER_POWER_MODE_AUTO_DEEPSLEEP_EXIT = 5,
};

int wpa_driver_mrvl_get_mac_addr(struct wpa_driver_wext_data *drv, char *buf, size_t buf_len)
{
    struct ifreq ifreq;
    char *mac;
    int ret = -1;
    //wpa_printf(MSG_DEBUG, "enter into %s", __FUNCTION__);

    memset(&ifreq, 0, sizeof(ifreq));
    strncpy(ifreq.ifr_name, drv->ifname, IFNAMSIZ);
    if (ioctl(drv->ioctl_sock, SIOCGIFHWADDR, &ifreq) < 0) {
        wpa_printf(MSG_ERROR, "ioctl[SIOCGIFHWADDR]");
        return ret;
    }
    mac = ifreq.ifr_hwaddr.sa_data;
    ret = snprintf(buf, buf_len, "Macaddr = " MACSTR "\n", MAC2STR(mac));

    return ret;
}

int wpa_driver_mrvl_get_rssi(struct wpa_driver_wext_data *drv, char *buf, size_t buf_len)
{
    struct iwreq iwr;
    char bssid[ETH_ALEN];
    int rssi;
    int ret = -1;

    //wpa_printf(MSG_DEBUG, "enter into %s", __FUNCTION__);

    memset(&iwr, 0, sizeof(iwr));
    strncpy(iwr.ifr_name, drv->ifname, IFNAMSIZ);

    if (ioctl(drv->ioctl_sock, SIOCGIWAP, &iwr) < 0) {
        wpa_printf(MSG_ERROR, "ioctl[SIOCGIWAP]");
        return ret;
    }
    memcpy(bssid, iwr.u.ap_addr.sa_data, ETH_ALEN);
    //get rssi
    memset(&iwr, 0, sizeof(iwr));
    strncpy(iwr.ifr_name, drv->ifname, IFNAMSIZ);
    *((int *)iwr.u.name) = WLANRSSI;
    *((int *)(iwr.u.name + 4)) = 1;//average beacon rssi
    if (ioctl(drv->ioctl_sock, WLAN_SETINT_GETINT, &iwr) < 0) {
        wpa_printf(MSG_DEBUG, "ioctl[WLAN_SETINT_GETINT]");
        return ret;
    }
    rssi = *((int *)iwr.u.name);
    wpa_printf(MSG_MSGDUMP, "rssi = %d", rssi);
    ret = snprintf(buf, buf_len, MACSTR " Rssi %d\n", MAC2STR(bssid), rssi);

    //remember rssi for roaming scan, it has moved to ctrl_iface.c
    //((struct wpa_supplicant*)drv->ctx)->rssi = rssi;

    return ret;
}

int wpa_driver_mrvl_get_linkspeed(struct wpa_driver_wext_data *drv, char *buf, size_t buf_len)
{
    struct iwreq iwr;
    double speed;
    int ret = -1;
    
    //wpa_printf(MSG_DEBUG, "enter into %s", __FUNCTION__);

    memset(&iwr, 0, sizeof(iwr));
    strncpy(iwr.ifr_name, drv->ifname, IFNAMSIZ);
    if (ioctl(drv->ioctl_sock, SIOCGIWRATE, &iwr) < 0) {
        wpa_printf(MSG_ERROR, "ioctl[SIOCGIWRATE]");
        return ret;
    }
    speed = (double)iwr.u.bitrate.value;
    speed = speed/1000000;
    ret = snprintf(buf, buf_len, "LinkSpeed %g\n", speed);
    return ret;
}

//active: 0 passive scan, 1 active scan
int wpa_driver_mrvl_set_scan_type(struct wpa_driver_wext_data *drv, int active)
{
    struct iwreq iwr;
    char buf[32];
    int ret = -1;

    memset(&iwr, 0, sizeof(iwr));
    strncpy(iwr.ifr_name, drv->ifname, IFNAMSIZ);
    memset(buf, 0, sizeof(buf));
    if(active){
        snprintf(buf, sizeof(buf), "active");
        iwr.u.data.length = 6;
    }else{
        snprintf(buf, sizeof(buf), "passive");
        iwr.u.data.length = 7;
    }
    iwr.u.data.pointer = (caddr_t) buf;
    if (ioctl(drv->ioctl_sock, WLANSCAN_TYPE, &iwr) < 0) {
        wpa_printf(MSG_ERROR, "ioctl[WLANSCAN_TYPE]");
        return ret;
    }
    ret = 0;
    return ret;
}

//wifi power control
static inline int
iw_set_ext(int			skfd,		/* Socket to the kernel */
	   const char *		ifname,		/* Device name */
	   int			    request,	/* WE ID */
	   struct iwreq *	pwrq)		/* Fixed part of the request */
{
  /* Set device name */
  strncpy(pwrq->ifr_name, ifname, IFNAMSIZ);
  /* Do the request */
  return(ioctl(skfd, request, pwrq));
}

static inline int
iw_get_ext(int			skfd,		/* Socket to the kernel */
	   const char *     ifname,		/* Device name */
	   int			    request,	/* WE ID */
	   struct iwreq*    piwr)		/* Fixed part of the request */
{
  strncpy(piwr->ifr_name, ifname, IFNAMSIZ);
  return(ioctl(skfd, request, piwr));
}

int wpa_driver_mrvl_powermode(void *drv, int mode)
{
#define ANDROID_DRIVER_POWER_MODE_AUTO_PS_ENABLED
//#undef  ANDROID_DRIVER_POWER_MODE_AUTO_PS_ENABLED
    struct iwreq iwr;
    int request_cmd;
    char buffer[16];
    int ret = -1;
    const char *ifname = ((struct wpa_driver_wext_data*)drv)->ifname;
    int skfd = ((struct wpa_driver_wext_data*)drv)->ioctl_sock;

    //wpa_printf(MSG_DEBUG, "enter into %s, mode is %d", __FUNCTION__, mode);

    memset(&iwr, 0, sizeof(iwr));
    switch (mode) {
        case ANDROID_DRIVER_POWER_MODE_AUTO:
#if defined(ANDROID_DRIVER_POWER_MODE_AUTO_PS_ENABLED)
            /* get power info */
            if ((ret = iw_get_ext(skfd, ifname, SIOCGIWPOWER, &iwr)) < 0) {
                wpa_printf(MSG_ERROR, "SIOCGIWPOWER erro");
                goto wpa_driver_wext_powermode_out;
            }
            request_cmd = SIOCSIWPOWER;
            iwr.u.power.flags = 0;
            iwr.u.power.disabled = 0;
            wpa_printf(MSG_INFO, "ANDROID_DRIVER_POWER_MODE_AUTO ps enabled");
#else
            request_cmd = SIOCSIWPOWER;
            iwr.u.power.disabled = 1;
            wpa_printf(MSG_INFO, "ANDROID_DRIVER_POWER_MODE_AUTO ps disabled");
#endif
            break;
        case ANDROID_DRIVER_POWER_MODE_ACTIVE:
            request_cmd = SIOCSIWPOWER;
            iwr.u.power.disabled = 1;
            wpa_printf(MSG_INFO, "ANDROID_DRIVER_POWER_MODE_ACTIVE ps disabled");
            break;
        case ANDROID_DRIVER_POWER_MODE_DEEPSLEEP:
            request_cmd = WLANDEEPSLEEP;
            buffer[0] = '1';
            iwr.u.data.pointer = (caddr_t) buffer;
            iwr.u.data.length = 1;
            wpa_printf(MSG_INFO, "ANDROID_DRIVER_POWER_MODE_DEEPSLEEP");
            break;
        case ANDROID_DRIVER_POWER_MODE_DEEPSLEEP_EXIT:
            request_cmd = WLANDEEPSLEEP;
            buffer[0] = '0';
            iwr.u.data.pointer = (caddr_t) buffer;
            iwr.u.data.length = 1;
            wpa_printf(MSG_INFO, "ANDROID_DRIVER_POWER_MODE_DEEPSLEEP_EXIT");
            break;
        case ANDROID_DRIVER_POWER_MODE_AUTO_DEEPSLEEP:
            request_cmd = WLAN_SETONEINT_GETONEINT;
            iwr.u.data.flags = WLAN_AUTODEEPSLEEP;
            iwr.u.data.pointer = (caddr_t) buffer;
            iwr.u.data.length = 1;
            ((int *) buffer)[0] = 1;
            wpa_printf(MSG_INFO, "ANDROID_DRIVER_POWER_MODE_AUTO_DEEPSLEEP");
            break;
        case ANDROID_DRIVER_POWER_MODE_AUTO_DEEPSLEEP_EXIT:
            request_cmd = WLAN_SETONEINT_GETONEINT;
            iwr.u.data.flags = WLAN_AUTODEEPSLEEP;
            iwr.u.data.pointer = (caddr_t) buffer;
            iwr.u.data.length = 1;
            ((int *) buffer)[0] = 0;
            wpa_printf(MSG_INFO, "ANDROID_DRIVER_POWER_MODE_AUTO_DEEPSLEEP_EXIT");
            break;
        default:
            wpa_printf(MSG_INFO, "Error: wpa_driver_wext_powermode");
            goto wpa_driver_wext_powermode_out;
            break;
    }

    ret = iw_set_ext(skfd, ifname, request_cmd, &iwr);
wpa_driver_wext_powermode_out:
    if (ret != 0) 
        wpa_printf(MSG_INFO, "in wpa_driver_wext_powermode, ioctl executes unsuccessfully");

    return ret;
}

//get driver supported channel list
int wpa_driver_mrvl_get_channel_list(struct wpa_driver_wext_data *drv, char *buf, size_t buf_len)
{
    struct iwreq iwr;
    struct iw_range *range;
    char buffer[sizeof(struct iw_range) * 2];	/* Large enough */
    char *pos, *end, *retpos;
    int i;
    int ret = -1;
    
#ifdef FOR_EMULATOR_DEBUG
    pos = buf;
    end = buf + buf_len;
    ret = snprintf(pos, end - pos, "channel : frequency\n");
    if (ret < 0 || ret >= end - pos)
        return pos - buf;
    pos += ret;
    for (i = 0; i < 11; i++) {
        retpos = pos;
        ret = snprintf(pos, end - pos, "%.3d : %.4d\n", i+1, (2412+i*5));
        if (ret < 0 || ret >= end - pos)
            return retpos - buf;
        pos += ret;		
    }
    wpa_printf(MSG_INFO,"%s",buf);
    return pos - buf;
#endif

    memset(&iwr, 0, sizeof(iwr));
    memset(buffer, 0, sizeof(buffer));
    strncpy(iwr.ifr_name, drv->ifname, IFNAMSIZ);

    iwr.u.data.pointer = (caddr_t) buffer;
    iwr.u.data.length = sizeof(buffer);
    iwr.u.data.flags = 0;

    if (ioctl(drv->ioctl_sock, SIOCGIWRANGE, &iwr) < 0) {
        wpa_printf(MSG_ERROR, "ioctl[SIOCGIWRANGE]");
        return ret;
    }
    range = (struct iw_range*)buffer; 
    pos = buf;
    end = buf + buf_len;
    ret = snprintf(pos, end - pos, "channel : frequency\n");
    if (ret < 0 || ret >= end - pos)
        return pos - buf;
    pos += ret;
    for (i = 0; i < range->num_frequency; i++) {
        retpos = pos;
        ret = snprintf(pos, end - pos, "%.3d : %.4d\n", range->freq[i].i, range->freq[i].m/100000);
        if (ret < 0 || ret >= end - pos)
            return retpos - buf;
        pos += ret;		
    }
    return pos - buf;
}

/**
 * get wlan tx total packets 
 */
int wpa_driver_mrvl_get_tx_packets(struct wpa_driver_wext_data *drv, int *packets, char *buf, size_t buf_len)
{
    struct iwreq iwr;
    int request_cmd;
    int buffer[1];
    int ret = -1;
    const char *ifname = drv->ifname;
    int skfd = drv->ioctl_sock;
    int subcmd;
    
    //wpa_printf(MSG_DEBUG, "enter into %s", __FUNCTION__);
    memset(buffer, 0, sizeof(buffer));
    memset(&iwr, 0, sizeof(iwr));

    request_cmd = WLAN_SET_GET_SIXTEEN_INT;
    iwr.u.data.flags = WLAN_GET_TX_PACKETS;
    iwr.u.data.pointer = (caddr_t) buffer;
    iwr.u.data.length = 1;

    ret = iw_set_ext(skfd, ifname, request_cmd, &iwr);
    
    if (ret < 0) {
        wpa_printf(MSG_INFO, "error : %s w_set_ext\n", __func__);
        return ret;
    }
    
    if (buf)
        ret = snprintf(buf, buf_len, "%d", buffer[0]);
    if (packets)
        *packets = buffer[0];

    return ret;
}

/**
 * get wlan tx total bytes 
 */
int wpa_driver_mrvl_get_tx_bytes(struct wpa_driver_wext_data *drv, char *buf, size_t buf_len)
{
    struct iwreq iwr;
    int request_cmd;
    int buffer[1];
    int ret = -1;
    const char *ifname = drv->ifname;
    int skfd = drv->ioctl_sock;
    int subcmd;
    
    memset(buffer, 0, sizeof(buffer));
    memset(&iwr, 0, sizeof(iwr));

    request_cmd = WLAN_SET_GET_SIXTEEN_INT;
    iwr.u.data.flags = WLAN_GET_TX_BYTES;
    iwr.u.data.pointer = (caddr_t) buffer;
    iwr.u.data.length = 1;

    ret = iw_set_ext(skfd, ifname, request_cmd, &iwr);
    
    if (ret < 0) {
        wpa_printf(MSG_INFO, "error : %s w_set_ext\n", __func__);
        return ret;
    }

    ret = snprintf(buf, buf_len, "%d", buffer[0]);

    return ret;
}

/**
 * get wlan tx total bytes 
 */
int wpa_driver_mrvl_set_suspend_lock(void *drv, int mod, char *buf, size_t buf_len)
{
    struct iwreq iwr;
    int request_cmd;
    int buffer[1];
    int ret = -1;
    const char *ifname = ((struct wpa_driver_wext_data *)drv)->ifname;
    int skfd = ((struct wpa_driver_wext_data *)drv)->ioctl_sock;
    int subcmd;
    
    memset(buffer, 0, sizeof(buffer));
    memset(&iwr, 0, sizeof(iwr));
    buffer[0] = mod;
    request_cmd = WLAN_SET_GET_SIXTEEN_INT;
    iwr.u.data.flags = WLAN_SET_SUSPEND_LOCK;
    iwr.u.data.pointer = (caddr_t) buffer;
    iwr.u.data.length = 1;

    ret = iw_set_ext(skfd, ifname, request_cmd, &iwr);
    
    if (ret < 0) {
        wpa_printf(MSG_INFO, "error : %s w_set_ext\n", __func__);
        return ret;
    }

    ret = snprintf(buf, buf_len, "%d", buffer[0]);

    return ret;
}

/**
 * feed wlan power management watchdog 
 */
int wpa_driver_mrvl_feed_pm_dog(void *drv, char *buf, size_t buf_len)
{
    struct iwreq iwr;
    int request_cmd;
    int buffer[1];
    int ret = -1;
    const char *ifname = ((struct wpa_driver_wext_data *)drv)->ifname;
    int skfd = ((struct wpa_driver_wext_data *)drv)->ioctl_sock;
    int subcmd;
    
    wpa_printf(MSG_INFO, "%s : start feed pm dog", "net_flow");
    memset(buffer, 0, sizeof(buffer));
    memset(&iwr, 0, sizeof(iwr));
    request_cmd = WLAN_SET_GET_SIXTEEN_INT;
    iwr.u.data.flags = WLAN_FEED_PM_DOG;
    iwr.u.data.pointer = (caddr_t) buffer;

    ret = iw_set_ext(skfd, ifname, request_cmd, &iwr);
    
    if (ret < 0) {
        wpa_printf(MSG_INFO, "error : %s w_set_ext\n", __func__);
        return ret;
    }

    wpa_printf(MSG_INFO, "%s : over feed pm dog", "net_flow");

    return ret;
}

int wpa_driver_mrvl_set_regioncode(struct wpa_driver_wext_data *drv, int regioncode)
{
    struct iwreq iwr;
    int ret = -1;

    memset(&iwr, 0, sizeof(iwr));
    strncpy(iwr.ifr_name, drv->ifname, IFNAMSIZ);
    *((int *)iwr.u.name) = WLANSETREGION;
    *((int *)(iwr.u.name + 4)) = regioncode;//average beacon rssi

    if (ioctl(drv->ioctl_sock, WLAN_SETONEINT_GETNONE, &iwr) < 0) {
        wpa_printf(MSG_ERROR, "ioctl[WLANSETREGION]");
        return ret;
    }
    ret = 0;
    return ret;
}

/**
 * hscfg command 
 */
int wpa_driver_mrvl_hscfg(void *drv, int val, char *buf, size_t buf_len)
{
    struct iwreq iwr;
    int request_cmd;
    int buffer[3];
    int ret = -1;
    const char *ifname = ((struct wpa_driver_wext_data *)drv)->ifname;
    int skfd = ((struct wpa_driver_wext_data *)drv)->ioctl_sock;
    int subcmd;
    
    memset(buffer, 0, sizeof(buffer));
    memset(&iwr, 0, sizeof(iwr));
    buffer[0] = val;
    //0xfe means GPIO 254 on Marvell 8688 chip.
    buffer[1] = 0xfe;
    //0xc8 means 200ms.
    buffer[2] = 0xc8;

    if (val == -1) {
        iwr.u.data.length = 1;
    } else {
        iwr.u.data.length = 3;
    }
    request_cmd = WLAN_SET_GET_SIXTEEN_INT;
    iwr.u.data.flags = WLANHSCFG;
    iwr.u.data.pointer = (caddr_t) buffer;

    ret = iw_set_ext(skfd, ifname, request_cmd, &iwr);
    
    if (ret < 0) {
        wpa_printf(MSG_INFO, "error : %s w_set_ext\n", __func__);
        return ret;
    }

    ret = snprintf(buf, buf_len, "%d", buffer[0]);

    return ret;
}

const unsigned char ip_base_index = 8;
const char  arpfileter_base[] =  {
       0x15,   //00: Type low
       0x01,   //01: Type high
       0x10,   //02: unknown, should be length low
       0x00,   //03: unknown, should be length high
       0x01,    //04: boardcast low
       0x00,     //05: boardcast high
       0x08,    //06: ARP  low
       0x06,    //07: ARP high
       0x00,     //08: IP1
       0x00,    //09: IP2
       0x00,    //10: IP3
       0x00,    //11: IP4
       0x02,    //12: unicase low
       0x00,    //13: unicase low
       0xff,      //14: ALL low
       0xff,      //15: ALL high
       0xff,      //16: IP1 
       0xff,      //17: IP2
       0xff,      //18: IP3
       0xff       //19: IP4
   };

 /**
  * set arpfilter filter for hot sleep mode
  */
 int wpa_driver_mrvl_set_arpfilter(void *drv, char * data, int data_len, char *buf, size_t buf_len)
 {
     struct iwreq iwr;
     int request_cmd;
     int ret = -1;
     const char *ifname = ((struct wpa_driver_wext_data *)drv)->ifname;
     int skfd = ((struct wpa_driver_wext_data *)drv)->ioctl_sock;
     
     wpa_printf(MSG_INFO, " set arpfilter" );
     
     request_cmd = WLANARPFILTER;

     memset(&iwr, 0, sizeof(iwr));
     strncpy(iwr.ifr_name, ifname, IFNAMSIZ);
     iwr.u.data.pointer = data;
     iwr.u.data.length = data_len;
     iwr.u.data.flags = 0;

 
     ret = iw_set_ext(skfd, ifname, request_cmd, &iwr);
     
     if (ret < 0) {
         wpa_printf(MSG_INFO, "error : %s w_set_ext\n", __func__);
         return ret;
     }
 
    wpa_printf(MSG_INFO, " set arpfilter success." );
 
     return ret;
 }
 

int wpa_driver_mrvl_check_addr(struct wpa_driver_wext_data *drv, char *buf, size_t buf_len)
{
    int skfd;
    struct ifreq    ifr;
    struct in_addr addr_value;
    int ret = -1;
    char * data;
    int data_len;
    int i;

    wpa_printf(MSG_INFO, "Enter wpa_driver_mrvl_check_addr, ifname is %s", drv->ifname);

    skfd = drv->ioctl_sock;

    memset(&ifr, 0, sizeof(struct ifreq));
    strncpy(ifr.ifr_name, drv->ifname, IFNAMSIZ);
    
    if (ioctl(drv->ioctl_sock, SIOCGIFADDR, &ifr) < 0) {
        wpa_printf(MSG_ERROR, "ioctl[SIOCGIFADDR]");
        return ret;
    }

    addr_value = ((struct sockaddr_in*)&(ifr.ifr_addr))->sin_addr;
    
    wpa_printf(MSG_INFO, "wpa_driver_mrvl_check_addr: Get addr is string(%s), value(0x%x).", inet_ntoa(addr_value),addr_value.s_addr);

    data_len = sizeof (arpfileter_base);
    data = (char *)malloc( data_len * sizeof( char ) );
    if( NULL == data )
    {
        wpa_printf(MSG_ERROR, "wpa_driver_mrvl_check_addr: malloc failed.");
        return ret;
    }
    
    memcpy(data, arpfileter_base, data_len);
    
    data[ip_base_index + 0] = addr_value.s_addr & 0x000000ff;
    data[ip_base_index + 1] = (addr_value.s_addr & 0x0000ff00)>>8;
    data[ip_base_index + 2] = (addr_value.s_addr & 0x00ff0000)>>16;
    data[ip_base_index + 3] = (addr_value.s_addr & 0xff000000)>>24;

    wpa_printf (MSG_INFO,  "wpa_driver_mrvl_check_addr, data_len is %d",data_len );

    for ( i = 0; i < data_len; i ++)
    {
        wpa_printf ( MSG_INFO,"%d: 0x%2x",i,data[i]);
    }
    
    ret = wpa_driver_mrvl_set_arpfilter(  drv,  data,data_len, buf, buf_len );
    ret = 0;

    free( data );
       
    return ret;
}

static int wifilock = 0;
static int hscfg_enable=0;

int wpa_driver_mrvl_get_wifilock( void *drv, char *buf, size_t buf_len )
{
    int ret = -1;
	
    ret = snprintf(buf, buf_len, "%d",wifilock);

    return ret;
}

int wpa_driver_vendor_driver_cmd(void *priv, char *cmd, char *buf, size_t buf_len)
{

    struct wpa_driver_wext_data *drv = (struct wpa_driver_wext_data *)priv;
    int ret = -1;

    wpa_printf(MSG_DEBUG, "%s %s", __func__, cmd);

    if (gDrvData == NULL) {
        gDrvData = drv;
    }

    if( strncasecmp(cmd, "start", 5) == 0 ) {
        wpa_printf(MSG_DEBUG,"Start command");
        wpa_printf(MSG_DEBUG, "CTRL_IFACE: wifilock is %d, set wifilock to %d", wifilock, 1 );
        wifilock = 1;
        ret = 0;
    }else if( strncasecmp(cmd, "stop", 4) == 0 ) {
        wpa_printf(MSG_DEBUG,"Stop command");
        wpa_printf(MSG_DEBUG, "CTRL_IFACE: wifilock is %d, set wifilock to %d", wifilock, 0 );
        wifilock = 0;
        ret = 0;
    }else if( strncasecmp(cmd, "macaddr", 7) == 0 ) {
        //wpa_printf(MSG_DEBUG,"Macaddr command");
        ret = wpa_driver_mrvl_get_mac_addr(drv, buf, buf_len);
    }else if( strncasecmp(cmd, "rssi", 4) == 0 ) {
        //wpa_printf(MSG_DEBUG,"RSSI command");
        ret = wpa_driver_mrvl_get_rssi(drv, buf, buf_len);
    }else if (strncasecmp(cmd, "linkspeed", 9) == 0) {
        //wpa_printf(MSG_DEBUG, "LinkSpeed command");
        ret = wpa_driver_mrvl_get_linkspeed(drv, buf, buf_len);
    }else if( strncasecmp(cmd, "scan-active", 11) == 0 ){
        //wpa_printf(MSG_DEBUG, "scan-active command");
        ret = wpa_driver_mrvl_set_scan_type(drv, 1);
    }else if( strncasecmp(cmd, "scan-passive", 12) == 0 ){
        //wpa_printf(MSG_DEBUG, "scan-passive command");
        ret = wpa_driver_mrvl_set_scan_type(drv, 0);
    } else if (strncasecmp(cmd, "powermode", 9) == 0) {
        //wpa_printf(MSG_DEBUG, "powermode command = \"%s\"", cmd);
        ret = wpa_driver_mrvl_powermode(drv, atoi(strchr(cmd, ' ')));
    }else if( strncasecmp(cmd, "channel-list", 12) == 0){
        //wpa_printf(MSG_DEBUG, "channel-list command");
        ret = wpa_driver_mrvl_get_channel_list(drv, buf, buf_len);
    } else if (strncasecmp(cmd, "txpackets", 9) == 0) {
        //wpa_printf(MSG_DEBUG, "txpackets command");
        ret = wpa_driver_mrvl_get_tx_packets(drv, NULL, buf, buf_len);
    } else if (strncasecmp(cmd, "txbytes", 7) == 0) {
        //wpa_printf(MSG_DEBUG, "txbytes command");
        ret = wpa_driver_mrvl_get_tx_bytes(drv, buf, buf_len);
    } else if (strncasecmp(cmd, "suspendlock", 11) == 0) {
        wpa_printf(MSG_DEBUG, "suspendlock command");
	 ret = wpa_driver_mrvl_set_suspend_lock(drv, atoi(strchr(cmd, ' ')), buf, buf_len);
    } else if (strncasecmp(cmd, "pmdog", 5) == 0) {
        //wpa_printf(MSG_DEBUG, "pmdog command");
        ret = wpa_driver_mrvl_feed_pm_dog(drv, buf, buf_len);
    } else if (strncasecmp(cmd, "regioncode", 10) == 0) {
        //wpa_printf(MSG_DEBUG, "regioncode command");
        ret = wpa_driver_mrvl_set_regioncode(drv, atoi(strchr(cmd, ' ')));
    } else if (strncasecmp(cmd, "hscfg", 5) == 0) {
        wpa_printf(MSG_DEBUG, "hscfg command");
        ret = wpa_driver_mrvl_hscfg(drv, atoi(strchr(cmd, ' ')), buf, buf_len);
    } else if (strncasecmp(cmd, "newaddr", 7) == 0) {
        wpa_printf(MSG_DEBUG, "newaddr command");
        ret = wpa_driver_mrvl_check_addr(drv, buf, buf_len);
    } else if (strncasecmp(cmd, "wifilock", 8) == 0) {
        wpa_printf(MSG_DEBUG, "wifilock command");
        ret = wpa_driver_mrvl_get_wifilock(drv, buf, buf_len);
    }
  
    return ret;
}

int wpa_driver_init_vendor_data(void *priv)
{
    if (gDrvData == NULL) {
        gDrvData = priv;
    }
    wpa_printf(MSG_DEBUG, "enter into %s", __FUNCTION__);
    
    return 0;
}

int wpa_driver_deinit_vendor_data(void *priv)
{
    if (gDrvData == NULL) {
        gDrvData = priv;
    }
    wpa_printf(MSG_DEBUG, "enter into %s", __FUNCTION__);

    return 0;
}

 static int wpa_driver_wext_mlme(struct wpa_driver_wext_data *drv,
                 const unsigned char *addr, int cmd, int reason_code)
 {
     struct iwreq iwr;
     struct iw_mlme mlme;
     int ret = 0;
 

     memset(&iwr, 0, sizeof(iwr));
     strncpy(iwr.ifr_name, drv->ifname, IFNAMSIZ);
     memset(&mlme, 0, sizeof(mlme));
     mlme.cmd = cmd;
     mlme.reason_code = reason_code;
     mlme.addr.sa_family = ARPHRD_ETHER;
     memcpy(mlme.addr.sa_data, addr, ETH_ALEN);
     iwr.u.data.pointer = (caddr_t) &mlme;
     iwr.u.data.length = sizeof(mlme);
 
#ifdef FOR_EMULATOR_DEBUG
     return 0;
#endif
 
     if (ioctl(drv->ioctl_sock, SIOCSIWMLME, &iwr) < 0) {
         perror("ioctl[SIOCSIWMLME]");
         ret = -1;
     }
 
     return ret;
 }


 void
wpa_supplicant_event_mrvl_hs_activated( void *priv )
{
    
//    wpa_msg(wpa_s, MSG_INFO, WPA_EVENT_HS_ACTIVATED "- mrv8688 chip");
    wpa_printf(MSG_INFO, "Should send  WPA_EVENT_HS_ACTIVATED - mrv8688 chip");
    
    
}

 void
wpa_supplicant_event_mrvl_hs_deactivated( void *priv  )
{
    
//    wpa_msg(wpa_s, MSG_INFO, WPA_EVENT_HS_DEACTIVATED "- mrv8688 chip");
    wpa_printf(MSG_INFO, "Should send  WPA_EVENT_HS_DEACTIVATED - mrv8688 chip");

    //added by derek, need send "hscfg -1" command to marvell 8688 wlan drvier
    //wpa_driver_vendor_driver_cmd(priv, MRVL_HS_CONFIG_DISABLE, NULL, 0);

    
}
#if 0

#define MRVL_HS_CONFIG_ENABLE "hscfg 7"
#define MRVL_HS_CONFIG_DISABLE "hscfg -1"

static void
wpa_supplicant_event_mrvl_late_suspend( void *priv, char * custom )
{
    
    struct wpa_driver_wext_data *drv = priv;
    struct iwreq iwr;
    char bssid[ETH_ALEN];

    int ret = -1;
    int max_count = 0;
    int connected = 0;

    if( NULL == custom )
        return;

    wpa_printf(MSG_INFO, "%s: custom string is [%s]",__FUNCTION__, custom);
    if( strncmp( custom, " STATUS=CONNECTED", 17 ) == 0){
        wpa_printf(MSG_INFO, "%s: STATUS=CONNECTED",__FUNCTION__ );
        connected = 1;
     }
    else{
        wpa_printf(MSG_INFO, "%s : STATUS=DISCONNECTED",__FUNCTION__ );
     }
       
        
    //get wifilock information from "DRIVER_START" and "DRIVER_STOP" commands.
    if (wifilock == 1&& connected == 1 ) {
        wpa_printf(MSG_DEBUG, "wifilock is TRUE");
        //added by derek, need send "hscfg 3 0xfe 0xc8" command to marvell 8688 wlan drvier
        wpa_driver_vendor_driver_cmd(priv, MRVL_HS_CONFIG_ENABLE, NULL, 0);
        hscfg_enable = 1;
    } 
    else {
        wpa_printf(MSG_DEBUG, "wifilock is FALSE");

retry_disconnecte_ap:
        
	// disconnecte ap
        memset(&iwr, 0, sizeof(iwr));
        strncpy(iwr.ifr_name, drv->ifname, IFNAMSIZ);

        if (ioctl(drv->ioctl_sock, SIOCGIWAP, &iwr) < 0) {
            wpa_printf(MSG_ERROR, "ioctl[SIOCGIWAP]");
            return ;
        }
        memcpy(bssid, iwr.u.ap_addr.sa_data, ETH_ALEN);

        wpa_printf(MSG_INFO, "%s :try to  IW_MLME_DEAUTH ", __FUNCTION__);
        wpa_driver_wext_mlme(drv, bssid, IW_MLME_DEAUTH, 3);

        usleep(500*1000);


        // exit autodeepsleep
        wpa_printf(MSG_INFO, "%s : AUTO_DEEPSLEEP_EXIT", __FUNCTION__ );
        wpa_driver_mrvl_powermode(drv, ANDROID_DRIVER_POWER_MODE_AUTO_DEEPSLEEP_EXIT);
        usleep(1*1000);//must have a delay between autodeepsleep exit and enter deepsleep command.

         // enter deepsleep
        wpa_printf(MSG_INFO, "%s : try to enter DEEPSLEEP", __FUNCTION__);
        ret = wpa_driver_mrvl_powermode(drv, ANDROID_DRIVER_POWER_MODE_DEEPSLEEP);
        if ((ret < 0) && (max_count++ < 10)) {
            wpa_printf(MSG_INFO, "%s : enter DEEPSLEEP retrying %d...",__FUNCTION__, max_count);
            sleep(1);
            goto retry_disconnecte_ap;
        }

       usleep(1*1000);
       wpa_printf(MSG_INFO, "%s : SET SUSPENDLOCK", __FUNCTION__ );
       wpa_driver_mrvl_set_suspend_lock(drv, 1, NULL, 0);
    }

    
}

 void
wpa_supplicant_event_mrvl_late_resume( void *priv  )
{
    
    struct wpa_driver_wext_data *drv = priv;
    
    //get wifilock information from "DRIVER_START" and "DRIVER_STOP" commands.
    if (hscfg_enable == 1) {
        wpa_printf(MSG_DEBUG, "wifilock is TRUE");
	        //added by derek, need send "hscfg -1" command to marvell 8688 wlan drvier
        wpa_driver_vendor_driver_cmd(priv, MRVL_HS_CONFIG_DISABLE, NULL, 0);
        hscfg_enable = 0;
    } 
    else {
        
        wpa_printf(MSG_DEBUG, "wifilock is FALSE");

        //exit deepsleep
        wpa_printf(MSG_INFO, "%s : DEEPSLEEP_EXIT", __FUNCTION__);
        wpa_driver_mrvl_powermode(drv, ANDROID_DRIVER_POWER_MODE_DEEPSLEEP_EXIT);

        usleep(1*1000);//must have a delay between autodeepsleep exit and enter deepsleep command.

        // enter autodeepsleep
        wpa_printf(MSG_INFO, "%s : AUTO_DEEPSLEEP", __FUNCTION__);
        wpa_driver_mrvl_powermode(drv, ANDROID_DRIVER_POWER_MODE_AUTO_DEEPSLEEP);

        //connect to ap
        // TODO
    }

    
}

#endif

 void
wpa_supplicant_event_mrvl_hs_wakeup( void *priv  )
{
    
//    wpa_msg(wpa_s, MSG_INFO, WPA_EVENT_HS_WAKEUP "- mrv8688 chip");
    wpa_printf(MSG_INFO, "Should send  WPA_EVENT_HS_WAKEUP - mrv8688 chip");
    
}

 void
wpa_supplicant_event_mrvl_deep_sleep_awake( void *priv )
{
    
//    wpa_msg(wpa_s, MSG_INFO, WPA_EVENT_DEEP_SLEEP_AWAKE "- mrv8688 chip");

    wpa_printf(MSG_INFO, "Should send WPA_EVENT_DEEP_SLEEP_AWAKE - mrv8688 chip");
    
}
 int wpa_driver_vendor_event_custom(void *priv, char *custom)
 {
     struct wpa_driver_wext_data *drv = (struct wpa_driver_wext_data *)priv;
     int ret = -1;
 
     if (strncmp(custom, "HS_ACTIVATED", 12) == 0) {
         wpa_supplicant_event_mrvl_hs_activated(priv);
     } else if (strncmp(custom, "HS_DEACTIVATED", 14) == 0) {
         wpa_supplicant_event_mrvl_hs_deactivated(priv);
     } else if (strncmp(custom, "HS_WAKEUP", 9) == 0) {
 //                    wpa_supplicant_event_mrvl_hs_wakeup(priv);
     } else if (strncmp(custom, "EVENT=DS_AWAKE", 14) == 0) {
 //                    wpa_supplicant_event_mrvl_deep_sleep_awake(priv);
     } else if (strncmp(custom, "EVENT=LATE_SUSPEND", 18) == 0) {
 //            wpa_supplicant_event_mrvl_late_suspend(priv,custom + 18 );
     } else if (strncmp(custom, "EVENT=LATE_RESUME", 17) == 0) {
 //            wpa_supplicant_event_mrvl_late_resume(priv);
     }
 
     return 0;
 }

