/** @file wapic/include/common.h
 *  @brief This header file contains data structures and function declarations of common
 *
 *  Copyright (C) 2001-2008, Iwncomm Ltd.
 */

#ifndef COMMON_H
#define COMMON_H

#ifdef __linux__
#include <endian.h>
#include <byteswap.h>
#endif
#ifdef __FreeBSD__
#include <sys/types.h>
#include <sys/endian.h>

/** Bitwise Swap:16bit */
#define bswap_16 bswap16
/** Bitwise Swap:32bit */
#define bswap_32 bswap32
#endif

#ifdef CONFIG_NATIVE_WINDOWS
#include <winsock.h>
#include <winsock2.h>

static inline int daemon(int nochdir, int noclose)
{
	printf("Windows - daemon() not supported yet\n");
	return -1;
}

static inline void sleep(int seconds)
{
	Sleep(seconds * 1000);
}

static inline void usleep(unsigned long usec)
{
	Sleep(usec / 1000);
}

#ifndef timersub
/** timersub  */
#define timersub(a, b, res) do { \
	(res)->tv_sec = (a)->tv_sec - (b)->tv_sec; \
	(res)->tv_usec = (a)->tv_usec - (b)->tv_usec; \
	if ((res)->tv_usec < 0) { \
		(res)->tv_sec--; \
		(res)->tv_usec += 1000000; \
	} \
} while (0)
#endif
/** Struct timezone */
struct timezone {
	/** minutes west */
	int  tz_minuteswest;
	/**  dst time */
	int  tz_dsttime;
};

int gettimeofday(struct timeval *tv, struct timezone *tz);

#endif /* CONFIG_NATIVE_WINDOWS */

#if defined(__CYGWIN__) || defined(CONFIG_NATIVE_WINDOWS)

static inline unsigned short wpa_swap_16(unsigned short v)
{
	return ((v & 0xff) << 8) | (v >> 8);
}

static inline unsigned int wpa_swap_32(unsigned int v)
{
	return ((v & 0xff) << 24) | ((v & 0xff00) << 8) |
		((v & 0xff0000) >> 8) | (v >> 24);
}

/** Convert Little Endian To Host:16bit */
#define le_to_host16(n) (n)
/** Convert Host To Little Endian:16bit */
#define host_to_le16(n) (n)
/** Convert Big Endian To Host:16bit */
#define be_to_host16(n) wpa_swap_16(n)
/** Convert Host To Big Endian:16bit */
#define host_to_be16(n) wpa_swap_16(n)
/** Convert Little Endian To Host:32bit */
#define le_to_host32(n) (n)
/** Convert Big Endian To Host:32bit */
#define be_to_host32(n) wpa_swap_32(n)
/** Convert Host To Big Endian:32bit */
#define host_to_be32(n) wpa_swap_32(n)

#else /* __CYGWIN__ */

#if __BYTE_ORDER == __LITTLE_ENDIAN
/** Convert Little Endian To Host:16bit */
#define le_to_host16(n) (n)
/** Convert Host To Little Endian:16bit */
#define host_to_le16(n) (n)
/** Convert Big Endian To Host:16bit */
#define be_to_host16(n) bswap_16(n)
/** Convert Host To Big Endian:16bit */
#define host_to_be16(n) bswap_16(n)
/** Convert Little Endian To Host:32bit */
#define le_to_host32(n) (n)
/** Convert Big Endian To Host:32bit */
#define be_to_host32(n) bswap_32(n)
/** Convert Host To Big Endian:32bit */
#define host_to_be32(n) bswap_32(n)
#elif __BYTE_ORDER == __BIG_ENDIAN
/** Convert Little Endian To Host:16bit */
#define le_to_host16(n) bswap_16(n)
/** Convert Host To Little Endian:16bit */
#define host_to_le16(n) bswap_16(n)
/** Convert Big Endian To Host:16bit */
#define be_to_host16(n) (n)
/** Convert Host To Big Endian:16bit */
#define host_to_be16(n) (n)
/** Convert Little Endian To Host:32bit */
#define le_to_host32(n) bswap_32(n)
/** Convert Big Endian To Host:32bit */
#define be_to_host32(n) (n)
/** Convert Host To Big Endian:32bit */
#define host_to_be32(n) (n)
#ifndef WORDS_BIGENDIAN
/** WORDS_BIGENDIAN */
#define WORDS_BIGENDIAN
#endif
#else
#error Could not determine CPU byte order
#endif

#endif /* __CYGWIN__ */


#ifndef ETH_ALEN
/** ETH_ALEN */
#define ETH_ALEN 6
#endif
#ifndef ETH_HLEN
/** ETH_HLEN */
#define ETH_HLEN 14
#endif

#include <stdint.h>
/** Unsigned long long integer */
typedef uint64_t u64;
/** Unsigned long integer */
typedef uint32_t u32;
/** Unsigned short integer */
typedef uint16_t u16;
/** Unsigned character */
typedef uint8_t u8;
/** Signed long long integer */
typedef int64_t s64;
/** Signed long integer */
typedef int32_t s32;
/** Short integer */
typedef int16_t s16;
/** Signed character*/
typedef int8_t s8;
#if 1
#ifndef CONFIG_CTRL_IFACE_UDP
#define CONFIG_CTRL_IFACE_UDP
#endif
#endif

#include <stdio.h>
/** Get Short */
#define	GETSHORT(frm, v) do { (v) = (((frm[0]) <<8) | (frm[1]))& 0xffff;} while (0)
/** Get Short1 */
#define	GETSHORT1(frm, v) do { (v) = (((frm[1]) <<8) | (frm[0]))& 0xffff;} while (0)
/** Set Short */
#define	SETSHORT(frm, v) do{(frm[0])=((v)>>8)&0xff;(frm[1])=((v))&0xff;}while(0)

int hexstr2bin(const char *hex, u8 *buf, size_t len);
char * rel2abs_path(const char *rel_path);
int hex2num(char c);


/** Debugging function - conditional printf and hex dump. Driver wrappers can
 *  use these for debugging purposes.
 */
enum { MSG_MSGDUMP, MSG_DEBUG, MSG_INFO, MSG_WARNING, MSG_ERROR };

void wpa_printf(int level, char *fmt, ...)
__attribute__ ((format (printf, 2, 3)));

void wpa_hexdump(int level, const char *title, const u8 *buf, size_t len);
void wpa_hexdump_key(int level, const char *title, const u8 *buf, size_t len);
void wpa_hexdump_ascii(int level, const char *title, const u8 *buf,
		       size_t len);

void wpa_hexdump_ascii_key(int level, const char *title, const u8 *buf,
			   size_t len);

/** WPA_ASSERT */
#ifdef EAPOL_TEST
#define WPA_ASSERT(a)						       \
	do {							       \
		if (!(a)) {					       \
			printf("WPA_ASSERT FAILED '" #a "' "	       \
			       "%s %s:%d\n",			       \
			       __FUNCTION__, __FILE__, __LINE__);      \
			exit(1);				       \
		}						       \
	} while (0)
#else
#define WPA_ASSERT(a) do { } while (0)
#endif

#endif /* COMMON_H */
