/** @file driver.h
 *  @brief This header file contains data structures and function declarations of driver
 *
 *  Copyright (C) 2001-2008, Iwncomm Ltd.
 *  Copyright (C) 2008-2009, Marvell International Ltd. 
 */

#ifndef DRIVER_H
#define DRIVER_H

#include "common.h"

/** Driver version */
#define WPA_SUPPLICANT_DRIVER_VERSION 2
/** Enumaration:wpa_alg */
typedef enum { WPA_ALG_NONE, WPA_ALG_WEP, WPA_ALG_TKIP, WPA_ALG_CCMP , WAPI_ALG_SMS4} wpa_alg;
/** Enumaration:wpa_cipher */
typedef enum { CIPHER_NONE, CIPHER_WEP40, CIPHER_TKIP, CIPHER_CCMP,
	       CIPHER_WEP104 ,CIPHER_SMS4} wpa_cipher;

/** Enumaration:wpa_key_mgmt */
typedef enum { KEY_MGMT_802_1X, KEY_MGMT_PSK, KEY_MGMT_NONE,
	       KEY_MGMT_802_1X_NO_WPA, KEY_MGMT_WPA_NONE } wpa_key_mgmt;
/** Authentication Alogrithm:Open system */
#define AUTH_ALG_OPEN_SYSTEM	0x01
/** Authentication Alogrithm:Shared key */
#define AUTH_ALG_SHARED_KEY	0x02
/** Authentication Alogrithm:Leap */
#define AUTH_ALG_LEAP		0x04
/** IEEE80211_MODE:INFRA */
#define IEEE80211_MODE_INFRA	0
/** IEEE80211_MODE:IBSS */
#define IEEE80211_MODE_IBSS	1
/** IEEE80211_CAP:ESS */
#define IEEE80211_CAP_ESS	0x0001
/** IEEE80211_CAP:IBSS */
#define IEEE80211_CAP_IBSS	0x0002
/** IEEE80211_CAP:PRIVACY */
#define IEEE80211_CAP_PRIVACY	0x0010
/** Max WPA IE length */
#define SSID_MAX_WPA_IE_LEN 40
/** Max WAPI IE length */
#define SSID_MAX_WAPI_IE_LEN 100

/** Structure: wpa_scan_result */
struct wpa_scan_result {
	/** BSSID */
	unsigned char bssid[ETH_ALEN];
	/** SSID length */
	size_t ssid_len;
	/** SSID */
	unsigned char ssid[32];
	/** WPA IE length */
	size_t wpa_ie_len;
	/** WPA IE */
	unsigned char wpa_ie[SSID_MAX_WPA_IE_LEN];
	/** RSN IE length */
	size_t rsn_ie_len;
	/** RSN IE */
	unsigned char rsn_ie[SSID_MAX_WPA_IE_LEN];
	/** WAPI IE length */
	size_t wapi_ie_len;
	/** WAPI IE */
	unsigned char wapi_ie[SSID_MAX_WAPI_IE_LEN];
	/** Frequency */
	int freq; /* MHz */
	/** Privacy */
	int caps; /* e.g. privacy */
	/** Signal quality */
	int qual; 
	/** Noise */
	int noise;
	/** Level */
	int level;
	/** Max Rate */
	int maxrate;
};

/** Parameters for associate driver_ops. */
struct wpa_driver_associate_params {
	/** BSSID of the selected AP */
	u8 *bssid;
	/** SSID */
	const u8 *ssid;
	/** SSID length */
	size_t ssid_len;
	/** frequency */
	int freq;
	/** WPA IE */
	u8 *wpa_ie;
	/** WPA IE length */
	size_t wpa_ie_len;
	/** pairwise cipher */
	wpa_cipher pairwise_suite;
	/** Group cipher */
	wpa_cipher group_suite;
	/** Key management */
	wpa_key_mgmt key_mgmt_suite;
	/** Bit field of AUTH_ALG */
	int auth_alg;
	/** IEEE80211_MODE_ */
	int mode; 
};

/** Structure:wpa_driver_capa */
struct wpa_driver_capa {
/** Key Management : WPA */
#define WPA_DRIVER_CAPA_KEY_MGMT_WPA		0x00000001
/** Key Management : WPA2 */
#define WPA_DRIVER_CAPA_KEY_MGMT_WPA2		0x00000002
/** Key Management : WPA-PSK */
#define WPA_DRIVER_CAPA_KEY_MGMT_WPA_PSK	0x00000004
/** Key Management : WPA2-PSK */
#define WPA_DRIVER_CAPA_KEY_MGMT_WPA2_PSK	0x00000008
/** Key Management : WPA-NONE */
#define WPA_DRIVER_CAPA_KEY_MGMT_WPA_NONE	0x00000010
	/** Key Management */
	unsigned int key_mgmt;
/** Encryption:WEP40 */
#define WPA_DRIVER_CAPA_ENC_WEP40	0x00000001
/** Encryption:WEP104 */
#define WPA_DRIVER_CAPA_ENC_WEP104	0x00000002
/** Encryption:TKIP */
#define WPA_DRIVER_CAPA_ENC_TKIP	0x00000004
/** Encryption:CCMP */
#define WPA_DRIVER_CAPA_ENC_CCMP	0x00000008
	/** Encryption */
	unsigned int enc;
/** Authentication:Open */
#define WPA_DRIVER_AUTH_OPEN		0x00000001
/** Authentication:Shared */
#define WPA_DRIVER_AUTH_SHARED		0x00000002
/** Authentication:LEAP */
#define WPA_DRIVER_AUTH_LEAP		0x00000004
	/** Authentication */
	unsigned int auth;

/** Driver generated WPA/RSN IE */
#define WPA_DRIVER_FLAGS_DRIVER_IE	0x00000001
/** Set keys after assoc */
#define WPA_DRIVER_FLAGS_SET_KEYS_AFTER_ASSOC 0x00000002
	/** Flag */
	unsigned int flags;
};

/** wpa_driver_ops structure */
struct wpa_driver_ops {
	/** name of the driver interface */
	const char *name;
	/** one line description of the driver interface */
	const char *desc;
	/** Function pointer to get the frequency */
	int (*get_freq)(void *priv, int *freq);
	/** Function pointer to set WPA IE */
	int (*set_wpa_ie)(void *priv, const u8 *wpa_ie, size_t wpa_ie_len);
	/** Function pointer to get the current BSSID */
	int (*get_bssid)(void *priv, u8 *bssid);

	/** Function pointer to get the current SSID */
	int (*get_ssid)(void *priv, u8 *ssid);

	/** Function pointer to enable/disable WPA support */
	int (*set_wpa)(void *priv, int enabled);

	/** Function pointer to configure encryption key */
	int (*set_key)(void *priv, wpa_alg alg, const u8 *addr,
		       int key_idx, int set_tx, const u8 *seq, size_t seq_len,
		       const u8 *key, size_t key_len);

	/** Function pointer to initialize driver interface */
	void * (*init)(void *ctx, const char *ifname);

	/** Function pointer to deinitialize driver interface */
	void (*deinit)(void *priv);

	/** Function pointer to enable/disable TKIP countermeasures */
	int (*set_countermeasures)(void *priv, int enabled);

	/** Function pointer to enable/disable unencrypted frame filtering */
	int (*set_drop_unencrypted)(void *priv, int enabled);

	/** Function pointer to request the driver to initiate scan */
	int (*scan)(void *priv, const u8 *ssid, size_t ssid_len);

	/** Function pointer to fetch the latest scan results */
	int (*get_scan_results)(void *priv,
				struct wpa_scan_result *results,
				size_t max_size);

	/** Function pointer to set authorized */
	int (*set_authorized)(void *priv, u8 *addr, int authorized);
	/** Function pointer to set wmode */
	int (*set_wmode)(void *priv, char *wmode, int len);

	/** Function pointer to request driver to deauthenticate */
	int (*deauthenticate)(void *priv, const u8 *addr, int reason_code);

	/** Function pointer to request driver to disassociate */
	int (*disassociate)(void *priv, const u8 *addr, int reason_code);

	/** Function pointer to request driver to associate */
	int (*associate)(void *priv,
			 struct wpa_driver_associate_params *params);

	/** Function pointer to set IEEE 802.11 authentication algorithm */
	int (*set_auth_alg)(void *priv, int auth_alg);

	/** Function pointer to add PMKSA cache entry to the driver */
	int (*add_pmkid)(void *priv, const u8 *bssid, const u8 *pmkid);

	/** Function pointer to remove PMKSA cache entry to the driver */
	int (*remove_pmkid)(void *priv, const u8 *bssid, const u8 *pmkid);

	/** Function pointer to flush PMKSA cache */
	int (*flush_pmkid)(void *priv);

	/** Function pointer to flush PMKSA cache */
	int (*get_capa)(void *priv, struct wpa_driver_capa *capa);

	/** Function pointer to poll driver for association information */
	void (*poll)(void *priv);

	/** Function pointer to get interface name */
	const char * (*get_ifname)(void *priv);

	/** Function pointer to get own MAC address */
	const u8 * (*get_mac_addr)(void *priv);
};

#endif /* DRIVER_H */
