/** @file l2_packet.h
 *  @brief This header file contains data structures and function declarations of l2_packet
 *
 *  Copyright (c) 2003-2005, Jouni Malinen <jkmaline@cc.hut.fi>
 *  Copyright (C) 2001-2008, Iwncomm Ltd.
 */

#ifndef L2_PACKET_H
#define L2_PACKET_H

/** MAC to string */
#define MAC2STR(a) (a)[0], (a)[1], (a)[2], (a)[3], (a)[4], (a)[5]
/** MAC string */
#define MACSTR "%02x:%02x:%02x:%02x:%02x:%02x"

#ifndef ETH_P_EAPOL
/** ETH_P_EAPOL */
#define ETH_P_EAPOL 0x888e
#endif

#ifndef ETH_P_RSN_PREAUTH
/**  ETH_P_RSN_PREAUTH */
#define ETH_P_RSN_PREAUTH 0x88c7
#endif

#ifndef ETH_P_WAI
/** ETH_P_WAI */
#define ETH_P_WAI 0x88b4
#endif

#ifndef ETH_P_WAI_PREAUTH
/**  ETH_P_WAI_PREAUTH */
#define ETH_P_WAI_PREAUTH 0x88b5
#endif

/**
 * struct l2_packet_data - Internal l2_packet data structure
 *
 * This structure is used by the l2_packet implementation to store its private
 * data. Other files use a pointer to this data when calling the l2_packet
 * functions, but the contents of this structure should not be used directly
 * outside l2_packet implementation.
 */
struct l2_packet_data;

/** Structure: l2_ethhdr */
struct l2_ethhdr {
	/** Destination */
	u8 h_dest[ETH_ALEN];
	/** Source */
	u8 h_source[ETH_ALEN];
	/** Proto */
	u16 h_proto;
} __attribute__ ((packed));

/**
 * l2_packet_init - Initialize l2_packet interface
 */
struct l2_packet_data * l2_packet_init(
	const char *ifname, const u8 *own_addr, unsigned short protocol,
	void (*rx_callback)(void *ctx, const u8 *src_addr,
			    const u8 *buf, size_t len),
	void *rx_callback_ctx, int l2_hdr);

/**
 * l2_packet_deinit - Deinitialize l2_packet interface
 */
void l2_packet_deinit(struct l2_packet_data *l2);

/**
 * l2_packet_get_own_addr - Get own layer 2 address
 */
int l2_packet_get_own_addr(struct l2_packet_data *l2, u8 *addr);

/**
 * l2_packet_send - Send a packet
 */
int l2_packet_send(struct l2_packet_data *l2, const u8 *dst_addr, u16 proto,
		   const u8 *buf, size_t len);

/**
 * l2_packet_get_ip_addr - Get the current IP address from the interface
 */
int l2_packet_get_ip_addr(struct l2_packet_data *l2, char *buf, size_t len);


/**
 * l2_packet_notify_auth_start - Notify l2_packet about start of authentication
 */
void l2_packet_notify_auth_start(struct l2_packet_data *l2);

#endif /* L2_PACKET_H */
