/** @file priv_netlink.h
 *  @brief This header file contains data structures and function declarations of priv_netlink
 *
 *  Copyright (c) 2002-2005, Jouni Malinen <jkmaline@cc.hut.fi>
 *  Copyright (C) 2001-2008, Iwncomm Ltd.
 */

#ifndef PRIV_NETLINK_H
#define PRIV_NETLINK_H

/* Private copy of needed Linux netlink/rtnetlink definitions.
 *
 * This should be replaced with user space header once one is available with C
 * library, etc..
 */

#ifndef IFLA_IFNAME
/** IFLA Interface name */
#define IFLA_IFNAME 3
#endif
#ifndef IFLA_WIRELESS
/** IFLA Wireless */
#define IFLA_WIRELESS 11
#endif

/** Netlink Route */
#define NETLINK_ROUTE 0
/** Netlink User Socket */
#define NETLINK_USERSOCK 2
/** RTM GRP link */
#define RTMGRP_LINK 1
/** RTM Base */
#define RTM_BASE 0x10
/** RTM New Link */
#define RTM_NEWLINK (RTM_BASE + 0)
/** RTM Delete Link */
#define RTM_DELLINK (RTM_BASE + 1)
/** NLMSG_ALIGNTO */
#define NLMSG_ALIGNTO 4
/** Netlink message Allign */
#define NLMSG_ALIGN(len) (((len) + NLMSG_ALIGNTO - 1) & ~(NLMSG_ALIGNTO - 1))
/** Netlink message Length */
#define NLMSG_LENGTH(len) ((len) + NLMSG_ALIGN(sizeof(struct nlmsghdr)))
/** Netlink message Data */
#define NLMSG_DATA(nlh) ((void*) (((char*) nlh) + NLMSG_LENGTH(0)))
/** RTA_ALIGNTO */
#define RTA_ALIGNTO 4
/** RTA message Allign */
#define RTA_ALIGN(len) (((len) + RTA_ALIGNTO - 1) & ~(RTA_ALIGNTO - 1))
/**  RTA_OK */
#define RTA_OK(rta,len) \
((len) > 0 && (rta)->rta_len >= sizeof(struct rtattr) && \
(rta)->rta_len <= (len))
/** RTA_NEXT */
#define RTA_NEXT(rta,attrlen) \
((attrlen) -= RTA_ALIGN((rta)->rta_len), \
(struct rtattr *) (((char *)(rta)) + RTA_ALIGN((rta)->rta_len)))

/** Structure:sockaddr_nl */
struct sockaddr_nl
{
	/** Socket Family */
	sa_family_t nl_family;
	/** Pad */
	unsigned short nl_pad;
	/** PID */
	u32 nl_pid;
	/** Groups */
	u32 nl_groups;
};

/** Message header */
struct nlmsghdr
{
	/** Message length */
	u32 nlmsg_len;
	/** Message type */
	u16 nlmsg_type;
	/** Message flags */
	u16 nlmsg_flags;
	/** Message sequence */
	u32 nlmsg_seq;
	/** Message PID */
	u32 nlmsg_pid;
};

/** Structure: ifinfomsg */
struct ifinfomsg
{
	/** Family */
	unsigned char ifi_family;
	/** Pad */
	unsigned char __ifi_pad;
	/** Type */
	unsigned short ifi_type;
	/** Index */
	int ifi_index;
	/** Flags */
	unsigned ifi_flags;
	/** Change */
	unsigned ifi_change;
};

/** Structure:rtattr */
struct rtattr
{
	/** Length */
	unsigned short rta_len;
	/** Type */
	unsigned short rta_type;
};

#endif /* PRIV_NETLINK_H */

