/** @file wapi_asue_i.h
 *  @brief This header file contains data structures and function declarations of wapi_asue_i
 *
 *  Copyright (c) 2003-2005, Jouni Malinen <jkmaline@cc.hut.fi>
 *  Copyright (C) 2001-2008, Iwncomm Ltd.
 */

#include "driver.h"

void wpa_supplicant_req_auth_timeout(struct wapi_asue_st *wpa_s,
				     int sec, int usec);
/* driver_ops */
static inline void * wpa_drv_init(struct wapi_asue_st *wpa_s,
				  const char *ifname)
{
	if (wpa_s->driver->init) {
		return wpa_s->driver->init(wpa_s, ifname);
	}
	return NULL;
}


static inline void wpa_drv_deinit(struct wapi_asue_st *wpa_s)
{
	if (wpa_s->driver->deinit)
		wpa_s->driver->deinit(wpa_s->drv_priv);
}


static inline int wpa_drv_set_drop_unencrypted(struct wapi_asue_st *wpa_s,
					       int enabled)
{
	if (wpa_s->driver->set_drop_unencrypted) {
		return wpa_s->driver->set_drop_unencrypted(wpa_s->drv_priv,
							   enabled);
	}
	return -1;
}

static inline int wpa_drv_set_countermeasures(struct wapi_asue_st *wpa_s,
					      int enabled)
{
	if (wpa_s->driver->set_countermeasures) {
		return wpa_s->driver->set_countermeasures(wpa_s->drv_priv,
							  enabled);
	}
	return -1;
}

static inline int wpa_drv_set_wpa(struct wapi_asue_st *wpa_s, int enabled)
{
	if (wpa_s->driver->set_wpa) {
		int ret = -1;
		ret = wpa_s->driver->set_wpa(wpa_s->drv_priv, enabled);
		wpa_printf(MSG_DEBUG, "WPA: in %s:%d,ret=%d", __func__, __LINE__,ret);
		return ret;
	}
	return -1;
}

static inline int wpa_drv_deauthenticate(struct wapi_asue_st *wpa_s,
					 const u8 *addr, int reason_code)
{
	if (wpa_s->driver->deauthenticate) {
		return wpa_s->driver->deauthenticate(wpa_s->drv_priv, addr,
						     reason_code);
	}
	return -1;
}

static inline int wpa_drv_disassociate(struct wapi_asue_st *wpa_s,
				       const u8 *addr, int reason_code)
{
	if (wpa_s->driver->disassociate) {
		return wpa_s->driver->disassociate(wpa_s->drv_priv, addr,
						   reason_code);
	}
	return -1;
}
static inline int wpa_drv_associate(struct wapi_asue_st *wpa_s,
				       struct wpa_driver_associate_params *params)
{
	if (wpa_s->driver->associate) {
		return wpa_s->driver->associate(wpa_s->drv_priv, params);
	}
	return -1;
}
static inline const char * wpa_drv_get_ifname(struct wapi_asue_st *wpa_s)
{
	if (wpa_s->driver->get_ifname) {
		return wpa_s->driver->get_ifname(wpa_s->drv_priv);
	}
	return NULL;
}

static inline const u8 * wpa_drv_get_mac_addr(struct wapi_asue_st *wpa_s)
{
	if (wpa_s->driver->get_mac_addr) {
		return wpa_s->driver->get_mac_addr(wpa_s->drv_priv);
	}
	return NULL;
}
static inline int wpa_drv_set_key(struct wapi_asue_st *wpa_s, wpa_alg alg,
				   const u8 *addr, int key_idx, int set_tx,
				   const u8 *seq, size_t seq_len,
				   const u8 *key, size_t key_len)
{
	if (wpa_s->driver->set_key) {
		return wpa_s->driver->set_key(wpa_s->drv_priv, alg, addr,
					      key_idx, set_tx, seq, seq_len,
					      key, key_len);
	}
	return -1;
}
static inline int
wpa_drv_set_authorized(struct wapi_asue_st *wpa_s, u8 *addr, int authorized)
{
	if (wpa_s->driver == NULL || wpa_s->driver->set_authorized== NULL||addr == NULL)
		return 0;
	return wpa_s->driver->set_authorized(wpa_s->drv_priv, addr,
						authorized);
}
static inline int
wpa_drv_set_wmode(struct wapi_asue_st *wpa_s, char *wmode, int len)
{
	if (wpa_s->driver == NULL || wpa_s->driver->set_wmode== NULL||wmode == NULL)
		return 0;
	return wpa_s->driver->set_wmode(wpa_s->drv_priv, wmode,
						len);
}
static inline int wpa_drv_get_scan_results(struct wapi_asue_st *wpa_s,
					   struct wpa_scan_result *results,
					   size_t max_size)
{
	if (wpa_s->driver->get_scan_results) {
		return wpa_s->driver->get_scan_results(wpa_s->drv_priv,
						       results, max_size);
	}
	return -1;
}
static inline int wpa_drv_get_bssid(struct wapi_asue_st *wpa_s, u8 *bssid)
{
	if (wpa_s->driver->get_bssid) {
		return wpa_s->driver->get_bssid(wpa_s->drv_priv, bssid);
	}
	return -1;
}
static inline int wpa_drv_scan(struct wapi_asue_st *wpa_s, const u8 *ssid,
			       size_t ssid_len)
{
	if (wpa_s->driver->scan) {
		return wpa_s->driver->scan(wpa_s->drv_priv, ssid, ssid_len);
	}
	return -1;
}


static inline int wpa_drv_set_wpa_ie(struct wapi_asue_st *wpa_s,  const u8 *wpa_ie, size_t wpa_ie_len)
{
	if (wpa_s->driver->set_wpa_ie) {
		return wpa_s->driver->set_wpa_ie(wpa_s->drv_priv, wpa_ie, wpa_ie_len);
	}
	return -1;
}
