/** @file wapi_config.h
 *  @brief This header file contains data structures and function declarations of wapi_config
 *
 *  Copyright (C) 2001-2008, Iwncomm Ltd.
 */

#ifndef __WAPI_CONFIG_H__
#define __WAPI_CONFIG_H__

/** Maximum SSID Length */
#define MAX_SSID_LEN 32
/** Block Length */
#define BK_LEN 16
/** PSK Length */
#define EAP_PSK_LEN 16
/** WAPI_CIPHER_NONE */
#define WAPI_CIPHER_NONE BIT(0)
/** WAPI_CIPHER_SMS4 */
#define WAPI_CIPHER_SMS4 BIT(4)

/** WAPI_KEY_MGMT_NONE */
#define WAPI_KEY_MGMT_NONE 0
/** WAPI_KEY_MGMT_CERT */
#define WAPI_KEY_MGMT_CERT BIT(1)
/** WAPI_KEY_MGMT_PSK */
#define WAPI_KEY_MGMT_PSK BIT(2)

/** WAPI config */
struct wapi_config{
	/** SSID */
	u8 *ssid;
	/** SSID length */
	size_t ssid_len;
	/** PSK */
	u8 *psk;
	/** PSK length */
	size_t psk_len;
	/** PSK bk */
	u8 psk_bk[BK_LEN];
	/** PSK set*/
	int psk_set;
	/** WAPI Policy */
	int wapi_policy;
	/** WPI policy*/
	int wpi_policy;
	/** Pairwise cipher */
	int pairwise_cipher;
	/** Group cipher */
	int group_cipher;
	/** Key management */
	int key_mgmt;
	/** Proto */
	int proto;
	/** Auth Algo */
	int auth_alg;
	/** Cert name */
	char *cert_name;
	/** Disabled flag */
	int disabled;
	//add by kaibing.chen
	char *ctrl_interface;
	char *ctrl_interface_group;
};
void wpa_config_free(struct wapi_config *config);
struct wapi_config * wpa_config_read(const char *config_file);

#endif	
