/** @file wapic/wapi.c
  * @brief This file contains functions for wapi
 *
 *  Copyright (C) 2001-2008, Iwncomm Ltd.
 */

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <assert.h>

/*for declaration of function `inet_addr'*/
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "include/common.h"
#include "include/wapi_asue.h"
#include "include/config.h"
#include "include/wapi_asue_retry.h"
#include "include/wapi_sm.h"
#include "include/wapi_asue_i.h"
#include "include/eloop.h"

static const int WAPI_SELECTOR_LEN = 4;
static const u16 WAPI_VERSION = 1;
static const u8 WAPI_AUTH_KEY_MGMT_CERT[] = { 0x00, 0x14, 0x72, 1 };
static const u8 WAPI_AUTH_KEY_MGMT_PSK[] = { 0x00, 0x14, 0x72, 2 };
static const u8 WAPI_CIPHER_SUITE_SMS4[] = { 0x00, 0x14, 0x72, 1 };

/** 
 *  @brief       memory copy  
 *
 *  @param dbuf    Destination buffer
 *  @param srcbuf  Source buffer
 *  @param len     Length of the buffer to be copied
 *  @return         None
 */
void * MEMCPY(void *dbuf, const void *srcbuf, int len) 
{
	memcpy(dbuf, srcbuf, len); 
	return dbuf+len;
}

/** 
 *  @brief       Free rx frag buffer  
 *
 *  @param frag  Point to structure wapi_rxfrag
 *  @return      None
 */
void *free_rxfrag(struct wapi_rxfrag *frag)
{
	if(frag != NULL){
		free_buffer((void *)(frag->data), frag->maxlen);
		free_buffer((void *)frag, sizeof(struct wapi_rxfrag ));
	}
	return NULL;
}

/** 
 *  @brief       Reset retry buffer  
 *
 *  @param wpa_s Point to structure wapi_asue_st
 *  @return      None
 */
void reset_rebuf(struct wapi_asue_st *wpa_s)
{
	wpa_s->buf.len = 0;
	if(wpa_s->buf.data)
	{
		free(wpa_s->buf.data);
		wpa_s->buf.data = NULL;
	}
}

/** 
 *  @brief       Copy data to retry buffer  
 *
 *  @param wpa_s        Point to structure wapi_asue_st
 *  @param sendto_ae    Point to data
 *  @param sendtoae_len Data length
 *  @return             None
 */
void set_rebuf(struct wapi_asue_st *wpa_s, u8* sendto_ae, int sendtoae_len)
{
	// fill sendinfo. used to check timeout
	if(wpa_s->buf.data !=NULL)
	{
		free((void *)(wpa_s->buf.data));
		wpa_s->buf.data = NULL;
	}
	wpa_s->buf.data = malloc(sendtoae_len);
	if(wpa_s->buf.data == NULL) return ;
	memset(wpa_s->buf.data, 0, sendtoae_len);
	memcpy(wpa_s->buf.data, sendto_ae, sendtoae_len);
	wpa_s->buf.len = sendtoae_len;
}
